[Match]
Name=<%= options[:device] %>

[Network]
Description=A basic static ethernet connection
Address=<%= options[:ip]%>/<%= options[:netmask] %>
<% if options[:gateway] -%>
Gateway=<%= options[:gateway] %>
<% end -%>
