#VAGRANT-BEGIN
# The contents below are automatically generated by Vagrant. Do not modify.
<% folders.each do |opts| %>
<%= opts[:name] %> <%= opts[:mount_point] %> <%= opts[:mount_type] %> <%= opts[:mount_options] || 'default' %> <%= opts[:dump] || 0 %> <%= opts[:fsck] || 0 %>
<%end%>
#VAGRANT-END