// Package client contains the Vagrant client implementation.
//
// The Vagrant client exposes a slightly higher level of abstraction
// than direct a API client for performing operations on an application.
// The primary consumer of this package is the CLI.
package client
