// Code generated by "stringer -type=HTTPMethod -linecomment ./downloader"; DO NOT EDIT.

package downloader

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[GET-0]
	_ = x[DELETE-1]
	_ = x[HEAD-2]
	_ = x[POST-3]
	_ = x[PUT-4]
}

const _HTTPMethod_name = "GETDELETEHEADPOSTPUT"

var _HTTPMethod_index = [...]uint8{0, 3, 9, 13, 17, 20}

func (i HTTPMethod) String() string {
	if i < 0 || i >= HTTPMethod(len(_HTTPMethod_index)-1) {
		return "HTTPMethod(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _HTTPMethod_name[_HTTPMethod_index[i]:_HTTPMethod_index[i+1]]
}
