#VAGRANT-BEGIN
# The contents below are automatically generated by Vagrant. Do not modify.
NM_CONTROLLED=<%= options.fetch(:nm_controlled, "no") %>
BOOTPROTO=static
ONBOOT=yes
DEVICE=<%= options[:device] %>
IPV6INIT=yes
IPV6ADDR=<%= options[:ip] %>/<%= options[:netmask] %>
<% if options[:gateway] -%>
IPV6_DEFAULTGW=<%= options[:gateway] %>
<% end %>
#VAGRANT-END
