#VAGRANT-BEGIN
# The contents below are automatically generated by Vagrant. Do not modify.
auto <%= options[:device] %>
iface <%= options[:device] %> inet dhcp
<% if !options[:use_dhcp_assigned_default_route] %>
    post-up ip route del default dev $IFACE || true
<% else %>
    # We need to disable eth0, see GH-2648
    post-up ip route del default dev <%= options[:root_device] %> || true
    post-up dhclient $IFACE
    pre-down ip route add default dev <%= options[:root_device] %>
<% end %>
#VAGRANT-END
