// Package flag is a thin layer over the stdlib flag package that provides
// some minimal features such as aliasing, autocompletion handling, improved
// defaults, etc. It was created for mitchellh/cli but can work as a standalone
// package.
package flag
