# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: plugin/grpc_broker.proto

require 'google/protobuf'

Google::Protobuf::DescriptorPool.generated_pool.build do
  add_file("plugin/grpc_broker.proto", :syntax => :proto3) do
    add_message "plugin.ConnInfo" do
      optional :service_id, :uint32, 1
      optional :network, :string, 2
      optional :address, :string, 3
    end
  end
end

module Plugin
  ConnInfo = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("plugin.ConnInfo").msgclass
end
