#VAGRANT-BEGIN
# The contents below are automatically generated by Vagrant. Do not modify.
auto eth<%= options[:interface] %>
iface eth<%= options[:interface] %> inet static
      address <%= options[:ip] %>
      netmask <%= options[:netmask] %>
#VAGRANT-END
