# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vagrant_plugin_sdk/plugin.proto

require 'google/protobuf'

require 'google/protobuf/any_pb'
require 'google/protobuf/timestamp_pb'
require 'google/protobuf/empty_pb'
require 'google/rpc/status_pb'
require 'protostructure_pb'
Google::Protobuf::DescriptorPool.generated_pool.build do
  add_file("vagrant_plugin_sdk/plugin.proto", :syntax => :proto3) do
    add_message "hashicorp.vagrant.sdk.Args" do
    end
    add_message "hashicorp.vagrant.sdk.Args.Seeds" do
      repeated :typed, :message, 1, "google.protobuf.Any"
      map :named, :string, :message, 2, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.Args.DataDir" do
    end
    add_message "hashicorp.vagrant.sdk.Args.DataDir.Basis" do
      optional :config_dir, :string, 1
      optional :cache_dir, :string, 2
      optional :data_dir, :string, 3
      optional :temp_dir, :string, 4
    end
    add_message "hashicorp.vagrant.sdk.Args.DataDir.Project" do
      optional :config_dir, :string, 1
      optional :cache_dir, :string, 2
      optional :data_dir, :string, 3
      optional :temp_dir, :string, 4
    end
    add_message "hashicorp.vagrant.sdk.Args.DataDir.Target" do
      optional :config_dir, :string, 1
      optional :cache_dir, :string, 2
      optional :data_dir, :string, 3
      optional :temp_dir, :string, 4
    end
    add_message "hashicorp.vagrant.sdk.Args.DataDir.Component" do
      optional :config_dir, :string, 1
      optional :cache_dir, :string, 2
      optional :data_dir, :string, 3
      optional :temp_dir, :string, 4
    end
    add_message "hashicorp.vagrant.sdk.Args.MetadataSet" do
      map :metadata, :string, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Args.Path" do
      optional :path, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Args.Folders" do
      optional :folders, :message, 1, "hashicorp.vagrant.sdk.Args.Hash"
    end
    add_message "hashicorp.vagrant.sdk.Args.TimeDuration" do
      optional :duration, :int32, 1
    end
    add_message "hashicorp.vagrant.sdk.Args.TerminalUI" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Logger" do
      optional :name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Args.JobInfo" do
      optional :local, :bool, 1
      optional :resource_id, :string, 2
      optional :resource_type, :enum, 3, "hashicorp.vagrant.sdk.Args.JobInfo.ResourceType"
      optional :id, :string, 4
    end
    add_enum "hashicorp.vagrant.sdk.Args.JobInfo.ResourceType" do
      value :BASIS, 0
      value :PROJECT, 1
      value :TARGET, 2
    end
    add_message "hashicorp.vagrant.sdk.Args.CorePluginManager" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.PluginManager" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Command" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Basis" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Project" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Provider" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Provisioner" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Target" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Target.State" do
      optional :state, :enum, 1, "hashicorp.vagrant.sdk.Args.Target.State.State"
    end
    add_enum "hashicorp.vagrant.sdk.Args.Target.State.State" do
      value :UNKNOWN, 0
      value :PENDING, 1
      value :CREATED, 2
      value :DESTROYED, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Target.Machine" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Target.Machine.State" do
      optional :id, :string, 1
      optional :short_description, :string, 2
      optional :long_description, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Box" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.BoxCollection" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.BoxMetadata" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.StateBag" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Host" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Guest" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Communicator" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Vagrantfile" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.Connection" do
    end
    add_message "hashicorp.vagrant.sdk.Args.Connection.SSHInfo" do
      proto3_optional :host, :string, 1
      proto3_optional :port, :string, 2
      proto3_optional :private_key_path, :string, 3
      proto3_optional :keys_only, :bool, 4
      proto3_optional :verify_host_key, :bool, 5
      proto3_optional :username, :string, 6
      proto3_optional :remote_user, :string, 7
      proto3_optional :compression, :bool, 8
      proto3_optional :dsa_authentication, :bool, 9
      proto3_optional :config, :string, 10
      repeated :extra_args, :string, 11
      proto3_optional :forward_agent, :bool, 12
      proto3_optional :forward_x11, :bool, 13
      repeated :forward_env, :string, 14
      optional :connect_timeout, :int64, 15
      proto3_optional :ssh_command, :string, 16
      proto3_optional :proxy_command, :string, 17
    end
    add_message "hashicorp.vagrant.sdk.Args.Connection.WinrmInfo" do
      optional :username, :string, 1
      optional :password, :string, 2
      optional :host, :string, 3
      optional :port, :int64, 4
      optional :guest_port, :int64, 5
      optional :max_tries, :int64, 6
      optional :retry_delay, :int64, 7
      optional :timeout, :int64, 8
      optional :transport, :enum, 9, "hashicorp.vagrant.sdk.Args.Connection.WinrmInfo.Transport"
      optional :ssl_peer_verification, :bool, 10
      optional :execution_time_limit, :string, 11
      optional :basic_auth_only, :bool, 12
      optional :codepage, :string, 13
    end
    add_enum "hashicorp.vagrant.sdk.Args.Connection.WinrmInfo.Transport" do
      value :NEGOTIATE, 0
      value :SSL, 1
      value :PLAINTEXT, 2
    end
    add_message "hashicorp.vagrant.sdk.Args.Push" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.SyncedFolder" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.TargetIndex" do
      optional :stream_id, :uint32, 1
      optional :network, :string, 2
      optional :addr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Args.NamedCapability" do
      optional :capability, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Args.ConfigData" do
      optional :source, :message, 1, "hashicorp.vagrant.sdk.Args.Class"
      optional :data, :message, 2, "hashicorp.vagrant.sdk.Args.Hash"
    end
    add_message "hashicorp.vagrant.sdk.Args.Direct" do
      repeated :arguments, :message, 1, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.Args.Array" do
      repeated :list, :message, 1, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.Args.HashEntry" do
      optional :key, :message, 1, "google.protobuf.Any"
      optional :value, :message, 2, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.Args.Hash" do
      repeated :entries, :message, 1, "hashicorp.vagrant.sdk.Args.HashEntry"
    end
    add_message "hashicorp.vagrant.sdk.Args.Class" do
      optional :name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Args.ProcRef" do
      optional :id, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Args.Symbol" do
      optional :str, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Args.Range" do
      optional :start, :int32, 1
      optional :end, :int32, 2
    end
    add_message "hashicorp.vagrant.sdk.Args.RubyLogger" do
      optional :name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Args.Set" do
      optional :list, :message, 1, "hashicorp.vagrant.sdk.Args.Array"
    end
    add_message "hashicorp.vagrant.sdk.Args.Options" do
      optional :options, :message, 1, "hashicorp.vagrant.sdk.Args.Hash"
    end
    add_message "hashicorp.vagrant.sdk.Args.Null" do
    end
    add_message "hashicorp.vagrant.sdk.Args.URL" do
      optional :url, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.FuncSpec" do
      optional :name, :string, 1
      repeated :args, :message, 2, "hashicorp.vagrant.sdk.FuncSpec.Value"
      repeated :result, :message, 3, "hashicorp.vagrant.sdk.FuncSpec.Value"
    end
    add_message "hashicorp.vagrant.sdk.FuncSpec.Value" do
      optional :name, :string, 1
      optional :type, :string, 2
      optional :value, :message, 3, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.FuncSpec.Args" do
      repeated :args, :message, 1, "hashicorp.vagrant.sdk.FuncSpec.Value"
    end
    add_message "hashicorp.vagrant.sdk.Auth" do
    end
    add_message "hashicorp.vagrant.sdk.Auth.AuthResponse" do
      optional :authenticated, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.ImplementsResp" do
      optional :implements, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI" do
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.IsInteractiveResponse" do
      optional :interactive, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.IsMachineReadableResponse" do
      optional :machine_readable, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.OutputRequest" do
      repeated :lines, :string, 1
      optional :style, :enum, 2, "hashicorp.vagrant.sdk.TerminalUI.OutputRequest.Style"
      optional :disable_new_line, :bool, 3
      optional :color, :string, 4
    end
    add_enum "hashicorp.vagrant.sdk.TerminalUI.OutputRequest.Style" do
      value :HEADER, 0
      value :ERROR, 1
      value :ERROR_BOLD, 2
      value :WARNING, 3
      value :WARNING_BOLD, 4
      value :INFO, 5
      value :SUCCESS, 6
      value :SUCCESS_BOLD, 7
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Response" do
      oneof :event do
        optional :input, :message, 1, "hashicorp.vagrant.sdk.TerminalUI.Event.InputResp"
      end
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Event" do
      oneof :event do
        optional :line, :message, 1, "hashicorp.vagrant.sdk.TerminalUI.Event.Line"
        optional :status, :message, 2, "hashicorp.vagrant.sdk.TerminalUI.Event.Status"
        optional :named_values, :message, 3, "hashicorp.vagrant.sdk.TerminalUI.Event.NamedValues"
        optional :raw, :message, 4, "hashicorp.vagrant.sdk.TerminalUI.Event.Raw"
        optional :table, :message, 5, "hashicorp.vagrant.sdk.TerminalUI.Event.Table"
        optional :step_group, :message, 6, "hashicorp.vagrant.sdk.TerminalUI.Event.StepGroup"
        optional :step, :message, 7, "hashicorp.vagrant.sdk.TerminalUI.Event.Step"
        optional :input, :message, 8, "hashicorp.vagrant.sdk.TerminalUI.Event.Input"
        optional :clear_line, :message, 9, "hashicorp.vagrant.sdk.TerminalUI.Event.ClearLine"
      end
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Event.Input" do
      optional :prompt, :string, 1
      optional :style, :string, 2
      optional :secret, :bool, 3
      optional :color, :string, 4
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Event.InputResp" do
      optional :input, :string, 1
      optional :error, :message, 2, "google.rpc.Status"
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Event.Status" do
      optional :status, :string, 1
      optional :msg, :string, 2
      optional :step, :bool, 3
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Event.Line" do
      optional :msg, :string, 1
      optional :style, :string, 2
      optional :disable_new_line, :bool, 3
      optional :color, :string, 4
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Event.Raw" do
      optional :data, :bytes, 1
      optional :stderr, :bool, 2
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Event.NamedValue" do
      optional :name, :string, 1
      optional :value, :string, 2
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Event.NamedValues" do
      repeated :values, :message, 1, "hashicorp.vagrant.sdk.TerminalUI.Event.NamedValue"
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Event.TableEntry" do
      optional :value, :string, 1
      optional :color, :string, 2
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Event.TableRow" do
      repeated :entries, :message, 1, "hashicorp.vagrant.sdk.TerminalUI.Event.TableEntry"
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Event.Table" do
      repeated :headers, :string, 1
      repeated :rows, :message, 2, "hashicorp.vagrant.sdk.TerminalUI.Event.TableRow"
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Event.StepGroup" do
      optional :close, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Event.Step" do
      optional :id, :int32, 1
      optional :close, :bool, 2
      optional :msg, :string, 3
      optional :status, :string, 4
      optional :output, :bytes, 5
    end
    add_message "hashicorp.vagrant.sdk.TerminalUI.Event.ClearLine" do
    end
    add_message "hashicorp.vagrant.sdk.Map" do
    end
    add_message "hashicorp.vagrant.sdk.Map.Request" do
      optional :args, :message, 1, "hashicorp.vagrant.sdk.FuncSpec.Args"
      optional :result, :string, 2
    end
    add_message "hashicorp.vagrant.sdk.Map.Response" do
      optional :result, :message, 1, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.Map.ListResponse" do
      repeated :funcs, :message, 1, "hashicorp.vagrant.sdk.FuncSpec"
    end
    add_message "hashicorp.vagrant.sdk.StateBag" do
    end
    add_message "hashicorp.vagrant.sdk.StateBag.GetRequest" do
      optional :key, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.StateBag.GetResponse" do
      optional :value, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.StateBag.GetOkResponse" do
      optional :ok, :bool, 1
      optional :value, :string, 2
    end
    add_message "hashicorp.vagrant.sdk.StateBag.PutRequest" do
      optional :key, :string, 1
      optional :value, :string, 2
    end
    add_message "hashicorp.vagrant.sdk.StateBag.PutResponse" do
    end
    add_message "hashicorp.vagrant.sdk.StateBag.RemoveRequest" do
      optional :key, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.StateBag.RemoveResponse" do
    end
    add_message "hashicorp.vagrant.sdk.PluginInfo" do
    end
    add_message "hashicorp.vagrant.sdk.PluginInfo.ComponentList" do
      repeated :component, :uint32, 1
    end
    add_message "hashicorp.vagrant.sdk.PluginInfo.Name" do
      optional :name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.PluginInfo.ComponentOptionsMap" do
      map :options, :uint32, :message, 1, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.PluginInfo.CommandOptions" do
      optional :primary, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.PluginInfo.ProviderOptions" do
      optional :priority, :int32, 1
      optional :parallel, :bool, 2
      optional :box_optional, :bool, 3
      optional :defaultable, :bool, 4
    end
    add_message "hashicorp.vagrant.sdk.PluginInfo.SyncedFolderOptions" do
      optional :priority, :int32, 1
    end
    add_message "hashicorp.vagrant.sdk.PluginManager" do
    end
    add_message "hashicorp.vagrant.sdk.PluginManager.PluginsRequest" do
      repeated :types, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.PluginManager.PluginsResponse" do
      repeated :plugins, :message, 1, "hashicorp.vagrant.sdk.PluginManager.Plugin"
    end
    add_message "hashicorp.vagrant.sdk.PluginManager.Plugin" do
      optional :name, :string, 1
      optional :type, :string, 2
      optional :plugin, :message, 3, "google.protobuf.Any"
      optional :options, :message, 4, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.CorePluginManager" do
    end
    add_message "hashicorp.vagrant.sdk.CorePluginManager.GetPluginRequest" do
      optional :type, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.CorePluginManager.GetPluginResponse" do
      optional :plugin, :message, 1, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.Provider" do
    end
    add_message "hashicorp.vagrant.sdk.Provider.UsableResp" do
      optional :is_usable, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.Provider.InstalledResp" do
      optional :is_installed, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.Provider.ActionRequest" do
      optional :name, :string, 1
      optional :func_args, :message, 2, "hashicorp.vagrant.sdk.FuncSpec.Args"
    end
    add_message "hashicorp.vagrant.sdk.Command" do
    end
    add_message "hashicorp.vagrant.sdk.Command.Flag" do
      optional :long_name, :string, 1
      optional :short_name, :string, 2
      optional :description, :string, 3
      optional :default_value, :string, 4
      optional :type, :enum, 5, "hashicorp.vagrant.sdk.Command.Flag.Type"
      repeated :aliases, :string, 6
    end
    add_enum "hashicorp.vagrant.sdk.Command.Flag.Type" do
      value :STRING, 0
      value :BOOL, 2
    end
    add_message "hashicorp.vagrant.sdk.Command.CommandInfo" do
      optional :name, :string, 1
      optional :help, :string, 2
      optional :synopsis, :string, 3
      repeated :flags, :message, 4, "hashicorp.vagrant.sdk.Command.Flag"
      repeated :subcommands, :message, 5, "hashicorp.vagrant.sdk.Command.CommandInfo"
      optional :primary, :bool, 6
    end
    add_message "hashicorp.vagrant.sdk.Command.CommandInfoResp" do
      optional :command_info, :message, 1, "hashicorp.vagrant.sdk.Command.CommandInfo"
    end
    add_message "hashicorp.vagrant.sdk.Command.ExecuteResp" do
      optional :exit_code, :int32, 1
    end
    add_message "hashicorp.vagrant.sdk.Command.ExecuteReq" do
      optional :spec, :message, 1, "hashicorp.vagrant.sdk.FuncSpec.Args"
      repeated :command_args, :string, 2
    end
    add_message "hashicorp.vagrant.sdk.Command.ExecuteSpecReq" do
      repeated :command_args, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Command.Arguments" do
      repeated :flags, :message, 1, "hashicorp.vagrant.sdk.Command.Arguments.Flag"
      repeated :args, :string, 2
    end
    add_message "hashicorp.vagrant.sdk.Command.Arguments.Flag" do
      optional :name, :string, 1
      optional :type, :enum, 4, "hashicorp.vagrant.sdk.Command.Arguments.Flag.Type"
      oneof :value do
        optional :string, :string, 2
        optional :bool, :bool, 3
      end
    end
    add_enum "hashicorp.vagrant.sdk.Command.Arguments.Flag.Type" do
      value :STRING, 0
      value :BOOL, 1
    end
    add_message "hashicorp.vagrant.sdk.Communicator" do
    end
    add_message "hashicorp.vagrant.sdk.Communicator.MatchResp" do
      optional :match, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.Communicator.ReadyResp" do
      optional :ready, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.Communicator.ExecuteResp" do
      optional :exit_code, :int32, 1
      optional :stdout, :string, 2
      optional :stderr, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Communicator.TestResp" do
      optional :valid, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.Communicator.Command" do
      optional :command, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Config" do
    end
    add_message "hashicorp.vagrant.sdk.Config.Merge" do
      optional :base, :message, 1, "hashicorp.vagrant.sdk.Args.ConfigData"
      optional :overlay, :message, 2, "hashicorp.vagrant.sdk.Args.ConfigData"
    end
    add_message "hashicorp.vagrant.sdk.Config.Finalize" do
      optional :config, :message, 1, "hashicorp.vagrant.sdk.Args.ConfigData"
    end
    add_message "hashicorp.vagrant.sdk.Config.Fields" do
      repeated :fields, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Config.Structure" do
      optional :struct, :message, 1, "protostructure.Struct"
    end
    add_message "hashicorp.vagrant.sdk.Config.RawRubyValue" do
      optional :source, :message, 1, "hashicorp.vagrant.sdk.Args.Class"
      optional :data, :message, 2, "hashicorp.vagrant.sdk.Args.Hash"
    end
    add_message "hashicorp.vagrant.sdk.Config.StructResponse" do
      oneof :value do
        optional :struct, :message, 1, "hashicorp.vagrant.sdk.Config.Structure"
        optional :raw, :bool, 2
      end
    end
    add_message "hashicorp.vagrant.sdk.Config.ConfigureRequest" do
      optional :json, :bytes, 1
    end
    add_message "hashicorp.vagrant.sdk.Config.RegisterResponse" do
      optional :identifier, :string, 1
      optional :scope, :string, 2
    end
    add_message "hashicorp.vagrant.sdk.Config.FieldDocumentation" do
      optional :name, :string, 1
      optional :synopsis, :string, 2
      optional :summary, :string, 3
      optional :optional, :bool, 4
      optional :env_var, :string, 5
      optional :type, :string, 6
      optional :default, :string, 7
    end
    add_message "hashicorp.vagrant.sdk.Config.MapperDocumentation" do
      optional :input, :string, 1
      optional :output, :string, 2
      optional :description, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Config.Documentation" do
      optional :description, :string, 1
      optional :example, :string, 2
      optional :input, :string, 3
      optional :output, :string, 4
      map :fields, :string, :message, 5, "hashicorp.vagrant.sdk.Config.FieldDocumentation"
      repeated :mappers, :message, 6, "hashicorp.vagrant.sdk.Config.MapperDocumentation"
    end
    add_message "hashicorp.vagrant.sdk.Platform" do
    end
    add_message "hashicorp.vagrant.sdk.Platform.DetectResp" do
      optional :detected, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.Platform.ParentResp" do
      optional :parent, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Platform.Capability" do
    end
    add_message "hashicorp.vagrant.sdk.Platform.Capability.NamedRequest" do
      optional :name, :string, 1
      optional :func_args, :message, 2, "hashicorp.vagrant.sdk.FuncSpec.Args"
    end
    add_message "hashicorp.vagrant.sdk.Platform.Capability.CheckResp" do
      optional :has_capability, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.Platform.Capability.Resp" do
      optional :result, :message, 1, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.SyncedFolder" do
    end
    add_message "hashicorp.vagrant.sdk.SyncedFolder.UsableResp" do
      optional :usable, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.Ref" do
    end
    add_message "hashicorp.vagrant.sdk.Ref.Box" do
      optional :resource_id, :string, 1
      optional :name, :string, 2
      optional :version, :string, 3
      optional :provider, :string, 4
    end
    add_message "hashicorp.vagrant.sdk.Ref.Basis" do
      optional :resource_id, :string, 1
      optional :path, :string, 2
      optional :name, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Ref.Project" do
      optional :resource_id, :string, 1
      optional :path, :string, 2
      optional :basis, :message, 3, "hashicorp.vagrant.sdk.Ref.Basis"
      optional :name, :string, 4
    end
    add_message "hashicorp.vagrant.sdk.Ref.Target" do
      optional :resource_id, :string, 1
      optional :project, :message, 2, "hashicorp.vagrant.sdk.Ref.Project"
      optional :name, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Ref.Machine" do
      optional :resource_id, :string, 1
      optional :project, :message, 2, "hashicorp.vagrant.sdk.Ref.Project"
      optional :name, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Basis" do
    end
    add_message "hashicorp.vagrant.sdk.Basis.ResourceIdResponse" do
      optional :resource_id, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Basis.DefaultProviderResponse" do
      optional :provider_name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Target" do
    end
    add_message "hashicorp.vagrant.sdk.Target.ResourceIdResponse" do
      optional :resource_id, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Target.RecordResponse" do
      optional :record, :message, 1, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.Target.NameResponse" do
      optional :name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Target.ProjectResponse" do
      optional :project, :message, 1, "hashicorp.vagrant.sdk.Ref.Project"
    end
    add_message "hashicorp.vagrant.sdk.Target.SetNameRequest" do
      optional :name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Target.VagrantfileNameResponse" do
      optional :name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Target.VagrantfilePathResponse" do
      optional :path, :message, 1, "hashicorp.vagrant.sdk.Args.Path"
    end
    add_message "hashicorp.vagrant.sdk.Target.UpdatedAtResponse" do
      optional :updated_at, :message, 1, "google.protobuf.Timestamp"
    end
    add_message "hashicorp.vagrant.sdk.Target.GetUUIDResponse" do
      optional :uuid, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Target.SetUUIDRequest" do
      optional :uuid, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Target.Machine" do
    end
    add_message "hashicorp.vagrant.sdk.Target.Machine.BoxResponse" do
      proto3_optional :box, :message, 1, "hashicorp.vagrant.sdk.Args.Box"
    end
    add_message "hashicorp.vagrant.sdk.Target.Machine.SetIDRequest" do
      optional :id, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Target.Machine.GetIDResponse" do
      optional :id, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Target.Machine.SetStateRequest" do
      optional :state, :message, 1, "hashicorp.vagrant.sdk.Args.Target.Machine.State"
    end
    add_message "hashicorp.vagrant.sdk.Target.Machine.GetStateResponse" do
      optional :state, :message, 1, "hashicorp.vagrant.sdk.Args.Target.Machine.State"
    end
    add_message "hashicorp.vagrant.sdk.Target.Machine.UIDResponse" do
      optional :user_id, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Target.Machine.SyncedFoldersResponse" do
      repeated :synced_folders, :message, 1, "hashicorp.vagrant.sdk.Target.Machine.SyncedFoldersResponse.MachineSyncedFolder"
    end
    add_message "hashicorp.vagrant.sdk.Target.Machine.SyncedFoldersResponse.Folder" do
      optional :source, :string, 1
      optional :destination, :string, 2
      optional :options, :message, 3, "hashicorp.vagrant.sdk.Args.Hash"
    end
    add_message "hashicorp.vagrant.sdk.Target.Machine.SyncedFoldersResponse.MachineSyncedFolder" do
      optional :plugin, :message, 1, "hashicorp.vagrant.sdk.Args.SyncedFolder"
      optional :folder, :message, 2, "hashicorp.vagrant.sdk.Target.Machine.SyncedFoldersResponse.Folder"
    end
    add_message "hashicorp.vagrant.sdk.Project" do
    end
    add_message "hashicorp.vagrant.sdk.Project.ActiveTargetsResponse" do
      repeated :targets, :message, 1, "hashicorp.vagrant.sdk.Args.Target"
    end
    add_message "hashicorp.vagrant.sdk.Project.ConfigResponse" do
      optional :vagrantfile, :message, 1, "hashicorp.vagrant.sdk.Vagrantfile.Vagrantfile"
    end
    add_message "hashicorp.vagrant.sdk.Project.CwdResponse" do
      optional :path, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Project.DefaultProviderRequest" do
      optional :check_usable, :bool, 1
      repeated :exclude, :string, 2
      optional :force_default, :bool, 3
      optional :machine_name, :string, 4
    end
    add_message "hashicorp.vagrant.sdk.Project.DefaultProviderResponse" do
      optional :provider_name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Project.HomeResponse" do
      optional :path, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Project.LocalDataResponse" do
      optional :path, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Project.PrimaryTargetNameResponse" do
      optional :name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Project.ResourceIdResponse" do
      optional :resource_id, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Project.TargetRequest" do
      optional :name, :string, 1
      optional :provider, :string, 2
    end
    add_message "hashicorp.vagrant.sdk.Project.TargetNamesResponse" do
      repeated :names, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Project.TargetIdsResponse" do
      repeated :ids, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Project.VagrantfileNameResponse" do
      optional :name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile" do
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.ValueRequest" do
      repeated :path, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.Serialized" do
      optional :json, :bytes, 1
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.BoxCollection" do
      optional :directory, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.NamespaceRequest" do
      optional :namespace, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.TargetRequest" do
      optional :name, :string, 1
      optional :provider, :string, 2
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.TargetResponse" do
      optional :target, :message, 1, "hashicorp.vagrant.sdk.Args.Target"
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.TargetConfigRequest" do
      optional :name, :string, 1
      optional :provider, :string, 2
      optional :validate_provider, :bool, 5
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.TargetConfigResponse" do
      optional :target_config, :message, 1, "hashicorp.vagrant.sdk.Vagrantfile.MachineConfig"
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.TargetNamesResponse" do
      repeated :names, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.PrimaryTargetNameResponse" do
      optional :name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.ConfigVM" do
      repeated :allowed_synced_folder_types, :string, 1
      optional :allow_fstab_modification, :bool, 2
      optional :allow_hosts_modificaion, :bool, 3
      optional :base_mac, :string, 4
      optional :base_address, :string, 5
      optional :boot_timeout, :int32, 6
      optional :box, :string, 7
      optional :ignore_box_vagrantfile, :bool, 8
      optional :box_check_update, :bool, 9
      repeated :box_url, :string, 10
      optional :box_server_url, :string, 11
      optional :box_version, :string, 12
      optional :box_download_ca_cert, :string, 13
      optional :box_download_ca_path, :string, 14
      optional :box_download_checksum, :string, 15
      optional :box_download_checksum_type, :string, 16
      optional :box_download_client_cert, :string, 17
      optional :box_download_insecure, :bool, 18
      optional :box_download_location_trusted, :bool, 19
      map :box_download_options, :string, :string, 20
      optional :communicator, :string, 21
      optional :graceful_halt_timeout, :int32, 22
      optional :guest, :string, 23
      optional :hostname, :string, 24
      optional :post_up_message, :string, 25
      repeated :usable_port_range, :int32, 26
      repeated :box_extra_download_options, :string, 27
      repeated :providers, :message, 29, "hashicorp.vagrant.sdk.Vagrantfile.Provider"
      repeated :networks, :message, 31, "hashicorp.vagrant.sdk.Vagrantfile.Network"
      repeated :provisioners, :message, 32, "hashicorp.vagrant.sdk.Vagrantfile.Provisioner"
      repeated :synced_folders, :message, 33, "hashicorp.vagrant.sdk.Vagrantfile.SyncedFolder"
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.ConfigVagrant" do
      optional :host, :string, 1
      repeated :plugins, :string, 2
      repeated :sensitive, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.GeneralConfig" do
      optional :type, :string, 1
      optional :config, :message, 2, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.MachineConfig" do
      optional :name, :string, 1
      optional :config_vm, :message, 2, "hashicorp.vagrant.sdk.Vagrantfile.ConfigVM"
      optional :config_vagrant, :message, 3, "hashicorp.vagrant.sdk.Vagrantfile.ConfigVagrant"
      repeated :plugin_configs, :message, 4, "hashicorp.vagrant.sdk.Vagrantfile.GeneralConfig"
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.Provisioner" do
      optional :name, :string, 1
      optional :type, :string, 2
      optional :before, :string, 3
      optional :after, :string, 4
      optional :communicator_required, :bool, 5
      optional :config, :message, 6, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.Provider" do
      optional :type, :string, 1
      optional :config, :message, 2, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.Network" do
      optional :type, :string, 1
      optional :id, :string, 2
      optional :config, :message, 3, "google.protobuf.Any"
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.SyncedFolder" do
      optional :source, :string, 1
      optional :destination, :string, 2
      optional :config, :message, 3, "google.protobuf.Any"
      optional :create, :bool, 4
      optional :disabled, :bool, 5
      proto3_optional :group, :string, 6
      optional :id, :string, 7
      repeated :mount_options, :string, 8
      proto3_optional :owner, :string, 9
      proto3_optional :type, :string, 10
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.PushConfig" do
      optional :name, :string, 1
      optional :config, :message, 2, "hashicorp.vagrant.sdk.Vagrantfile.GeneralConfig"
    end
    add_message "hashicorp.vagrant.sdk.Vagrantfile.Vagrantfile" do
      optional :path, :string, 1
      optional :raw, :string, 2
      optional :current_version, :string, 3
      repeated :machine_configs, :message, 4, "hashicorp.vagrant.sdk.Vagrantfile.MachineConfig"
      repeated :push_configs, :message, 5, "hashicorp.vagrant.sdk.Vagrantfile.PushConfig"
      repeated :provisioners, :message, 6, "hashicorp.vagrant.sdk.Vagrantfile.Provisioner"
    end
    add_message "hashicorp.vagrant.sdk.TargetIndex" do
    end
    add_message "hashicorp.vagrant.sdk.TargetIndex.TargetIdentifier" do
      optional :id, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.TargetIndex.AllResponse" do
      repeated :targets, :message, 1, "hashicorp.vagrant.sdk.Args.Target"
    end
    add_message "hashicorp.vagrant.sdk.TargetIndex.IncludesResponse" do
      optional :exists, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.Box" do
    end
    add_message "hashicorp.vagrant.sdk.Box.AutomaticUpdateCheckAllowedResponse" do
      optional :allowed, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.Box.HasUpdateRequest" do
      optional :version, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Box.HasUpdateResponse" do
      optional :has_update, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.Box.UpdateInfoResponse" do
      optional :has_update, :bool, 1
      optional :metadata, :message, 2, "hashicorp.vagrant.sdk.Args.BoxMetadata"
      optional :new_version, :string, 3
      optional :new_provider, :string, 4
    end
    add_message "hashicorp.vagrant.sdk.Box.InUseResponse" do
      optional :in_use, :bool, 1
    end
    add_message "hashicorp.vagrant.sdk.Box.MachinesResponse" do
      repeated :machines, :message, 1, "hashicorp.vagrant.sdk.Args.Target.Machine"
    end
    add_message "hashicorp.vagrant.sdk.Box.BoxMetadataResponse" do
      optional :metadata, :message, 1, "hashicorp.vagrant.sdk.Args.Hash"
    end
    add_message "hashicorp.vagrant.sdk.Box.MetadataUrlResponse" do
      optional :metadata_url, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Box.NameResponse" do
      optional :name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Box.ProviderResponse" do
      optional :provider, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Box.VersionResponse" do
      optional :version, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.Box.EqualityResponse" do
      optional :result, :int32, 1
    end
    add_message "hashicorp.vagrant.sdk.BoxCollection" do
    end
    add_message "hashicorp.vagrant.sdk.BoxCollection.AddRequest" do
      optional :path, :message, 1, "hashicorp.vagrant.sdk.Args.Path"
      optional :name, :string, 2
      optional :version, :string, 3
      optional :metadataUrl, :string, 4
      optional :force, :bool, 5
      repeated :providers, :string, 6
    end
    add_message "hashicorp.vagrant.sdk.BoxCollection.AllResponse" do
      repeated :boxes, :message, 1, "hashicorp.vagrant.sdk.Args.Box"
    end
    add_message "hashicorp.vagrant.sdk.BoxCollection.CleanRequest" do
      optional :name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.BoxCollection.FindRequest" do
      optional :name, :string, 1
      optional :version, :string, 2
      repeated :providers, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.BoxMetadata" do
    end
    add_message "hashicorp.vagrant.sdk.BoxMetadata.BoxMetadataOpts" do
      optional :name, :string, 1
      optional :url, :string, 2
      optional :checksum, :string, 3
      optional :checksum_type, :string, 4
    end
    add_message "hashicorp.vagrant.sdk.BoxMetadata.LoadMetadataRequest" do
      optional :url, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.BoxMetadata.NameResponse" do
      optional :name, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.BoxMetadata.VersionQuery" do
      optional :version, :string, 1
      repeated :opts, :message, 2, "hashicorp.vagrant.sdk.BoxMetadata.BoxMetadataOpts"
    end
    add_message "hashicorp.vagrant.sdk.BoxMetadata.VersionResponse" do
      optional :version, :string, 1
      optional :status, :string, 2
      optional :description, :string, 3
    end
    add_message "hashicorp.vagrant.sdk.BoxMetadata.ListVersionsQuery" do
      repeated :opts, :message, 1, "hashicorp.vagrant.sdk.BoxMetadata.BoxMetadataOpts"
    end
    add_message "hashicorp.vagrant.sdk.BoxMetadata.ListVersionsResponse" do
      repeated :versions, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.BoxMetadata.ProviderRequest" do
      optional :version, :string, 1
      optional :name, :string, 2
    end
    add_message "hashicorp.vagrant.sdk.BoxMetadata.ProviderResponse" do
      optional :name, :string, 1
      optional :url, :string, 2
      optional :checksum, :string, 3
      optional :checksum_type, :string, 4
    end
    add_message "hashicorp.vagrant.sdk.BoxMetadata.ListProvidersRequest" do
      optional :version, :string, 1
    end
    add_message "hashicorp.vagrant.sdk.BoxMetadata.ListProvidersResponse" do
      repeated :providers, :string, 1
    end
  end
end

module Hashicorp
  module Vagrant
    module Sdk
      Args = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args").msgclass
      Args::Seeds = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Seeds").msgclass
      Args::DataDir = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.DataDir").msgclass
      Args::DataDir::Basis = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.DataDir.Basis").msgclass
      Args::DataDir::Project = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.DataDir.Project").msgclass
      Args::DataDir::Target = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.DataDir.Target").msgclass
      Args::DataDir::Component = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.DataDir.Component").msgclass
      Args::MetadataSet = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.MetadataSet").msgclass
      Args::Path = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Path").msgclass
      Args::Folders = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Folders").msgclass
      Args::TimeDuration = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.TimeDuration").msgclass
      Args::TerminalUI = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.TerminalUI").msgclass
      Args::Logger = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Logger").msgclass
      Args::JobInfo = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.JobInfo").msgclass
      Args::JobInfo::ResourceType = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.JobInfo.ResourceType").enummodule
      Args::CorePluginManager = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.CorePluginManager").msgclass
      Args::PluginManager = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.PluginManager").msgclass
      Args::Command = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Command").msgclass
      Args::Basis = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Basis").msgclass
      Args::Project = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Project").msgclass
      Args::Provider = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Provider").msgclass
      Args::Provisioner = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Provisioner").msgclass
      Args::Target = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Target").msgclass
      Args::Target::State = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Target.State").msgclass
      Args::Target::State::State = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Target.State.State").enummodule
      Args::Target::Machine = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Target.Machine").msgclass
      Args::Target::Machine::State = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Target.Machine.State").msgclass
      Args::Box = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Box").msgclass
      Args::BoxCollection = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.BoxCollection").msgclass
      Args::BoxMetadata = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.BoxMetadata").msgclass
      Args::StateBag = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.StateBag").msgclass
      Args::Host = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Host").msgclass
      Args::Guest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Guest").msgclass
      Args::Communicator = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Communicator").msgclass
      Args::Vagrantfile = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Vagrantfile").msgclass
      Args::Connection = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Connection").msgclass
      Args::Connection::SSHInfo = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Connection.SSHInfo").msgclass
      Args::Connection::WinrmInfo = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Connection.WinrmInfo").msgclass
      Args::Connection::WinrmInfo::Transport = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Connection.WinrmInfo.Transport").enummodule
      Args::Push = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Push").msgclass
      Args::SyncedFolder = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.SyncedFolder").msgclass
      Args::TargetIndex = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.TargetIndex").msgclass
      Args::NamedCapability = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.NamedCapability").msgclass
      Args::ConfigData = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.ConfigData").msgclass
      Args::Direct = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Direct").msgclass
      Args::Array = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Array").msgclass
      Args::HashEntry = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.HashEntry").msgclass
      Args::Hash = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Hash").msgclass
      Args::Class = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Class").msgclass
      Args::ProcRef = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.ProcRef").msgclass
      Args::Symbol = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Symbol").msgclass
      Args::Range = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Range").msgclass
      Args::RubyLogger = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.RubyLogger").msgclass
      Args::Set = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Set").msgclass
      Args::Options = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Options").msgclass
      Args::Null = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.Null").msgclass
      Args::URL = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Args.URL").msgclass
      FuncSpec = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.FuncSpec").msgclass
      FuncSpec::Value = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.FuncSpec.Value").msgclass
      FuncSpec::Args = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.FuncSpec.Args").msgclass
      Auth = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Auth").msgclass
      Auth::AuthResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Auth.AuthResponse").msgclass
      ImplementsResp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.ImplementsResp").msgclass
      TerminalUI = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI").msgclass
      TerminalUI::IsInteractiveResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.IsInteractiveResponse").msgclass
      TerminalUI::IsMachineReadableResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.IsMachineReadableResponse").msgclass
      TerminalUI::OutputRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.OutputRequest").msgclass
      TerminalUI::OutputRequest::Style = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.OutputRequest.Style").enummodule
      TerminalUI::Response = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Response").msgclass
      TerminalUI::Event = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Event").msgclass
      TerminalUI::Event::Input = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Event.Input").msgclass
      TerminalUI::Event::InputResp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Event.InputResp").msgclass
      TerminalUI::Event::Status = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Event.Status").msgclass
      TerminalUI::Event::Line = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Event.Line").msgclass
      TerminalUI::Event::Raw = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Event.Raw").msgclass
      TerminalUI::Event::NamedValue = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Event.NamedValue").msgclass
      TerminalUI::Event::NamedValues = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Event.NamedValues").msgclass
      TerminalUI::Event::TableEntry = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Event.TableEntry").msgclass
      TerminalUI::Event::TableRow = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Event.TableRow").msgclass
      TerminalUI::Event::Table = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Event.Table").msgclass
      TerminalUI::Event::StepGroup = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Event.StepGroup").msgclass
      TerminalUI::Event::Step = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Event.Step").msgclass
      TerminalUI::Event::ClearLine = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TerminalUI.Event.ClearLine").msgclass
      Map = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Map").msgclass
      Map::Request = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Map.Request").msgclass
      Map::Response = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Map.Response").msgclass
      Map::ListResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Map.ListResponse").msgclass
      StateBag = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.StateBag").msgclass
      StateBag::GetRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.StateBag.GetRequest").msgclass
      StateBag::GetResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.StateBag.GetResponse").msgclass
      StateBag::GetOkResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.StateBag.GetOkResponse").msgclass
      StateBag::PutRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.StateBag.PutRequest").msgclass
      StateBag::PutResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.StateBag.PutResponse").msgclass
      StateBag::RemoveRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.StateBag.RemoveRequest").msgclass
      StateBag::RemoveResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.StateBag.RemoveResponse").msgclass
      PluginInfo = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.PluginInfo").msgclass
      PluginInfo::ComponentList = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.PluginInfo.ComponentList").msgclass
      PluginInfo::Name = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.PluginInfo.Name").msgclass
      PluginInfo::ComponentOptionsMap = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.PluginInfo.ComponentOptionsMap").msgclass
      PluginInfo::CommandOptions = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.PluginInfo.CommandOptions").msgclass
      PluginInfo::ProviderOptions = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.PluginInfo.ProviderOptions").msgclass
      PluginInfo::SyncedFolderOptions = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.PluginInfo.SyncedFolderOptions").msgclass
      PluginManager = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.PluginManager").msgclass
      PluginManager::PluginsRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.PluginManager.PluginsRequest").msgclass
      PluginManager::PluginsResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.PluginManager.PluginsResponse").msgclass
      PluginManager::Plugin = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.PluginManager.Plugin").msgclass
      CorePluginManager = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.CorePluginManager").msgclass
      CorePluginManager::GetPluginRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.CorePluginManager.GetPluginRequest").msgclass
      CorePluginManager::GetPluginResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.CorePluginManager.GetPluginResponse").msgclass
      Provider = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Provider").msgclass
      Provider::UsableResp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Provider.UsableResp").msgclass
      Provider::InstalledResp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Provider.InstalledResp").msgclass
      Provider::ActionRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Provider.ActionRequest").msgclass
      Command = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Command").msgclass
      Command::Flag = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Command.Flag").msgclass
      Command::Flag::Type = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Command.Flag.Type").enummodule
      Command::CommandInfo = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Command.CommandInfo").msgclass
      Command::CommandInfoResp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Command.CommandInfoResp").msgclass
      Command::ExecuteResp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Command.ExecuteResp").msgclass
      Command::ExecuteReq = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Command.ExecuteReq").msgclass
      Command::ExecuteSpecReq = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Command.ExecuteSpecReq").msgclass
      Command::Arguments = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Command.Arguments").msgclass
      Command::Arguments::Flag = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Command.Arguments.Flag").msgclass
      Command::Arguments::Flag::Type = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Command.Arguments.Flag.Type").enummodule
      Communicator = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Communicator").msgclass
      Communicator::MatchResp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Communicator.MatchResp").msgclass
      Communicator::ReadyResp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Communicator.ReadyResp").msgclass
      Communicator::ExecuteResp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Communicator.ExecuteResp").msgclass
      Communicator::TestResp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Communicator.TestResp").msgclass
      Communicator::Command = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Communicator.Command").msgclass
      Config = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Config").msgclass
      Config::Merge = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Config.Merge").msgclass
      Config::Finalize = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Config.Finalize").msgclass
      Config::Fields = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Config.Fields").msgclass
      Config::Structure = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Config.Structure").msgclass
      Config::RawRubyValue = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Config.RawRubyValue").msgclass
      Config::StructResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Config.StructResponse").msgclass
      Config::ConfigureRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Config.ConfigureRequest").msgclass
      Config::RegisterResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Config.RegisterResponse").msgclass
      Config::FieldDocumentation = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Config.FieldDocumentation").msgclass
      Config::MapperDocumentation = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Config.MapperDocumentation").msgclass
      Config::Documentation = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Config.Documentation").msgclass
      Platform = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Platform").msgclass
      Platform::DetectResp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Platform.DetectResp").msgclass
      Platform::ParentResp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Platform.ParentResp").msgclass
      Platform::Capability = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Platform.Capability").msgclass
      Platform::Capability::NamedRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Platform.Capability.NamedRequest").msgclass
      Platform::Capability::CheckResp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Platform.Capability.CheckResp").msgclass
      Platform::Capability::Resp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Platform.Capability.Resp").msgclass
      SyncedFolder = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.SyncedFolder").msgclass
      SyncedFolder::UsableResp = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.SyncedFolder.UsableResp").msgclass
      Ref = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Ref").msgclass
      Ref::Box = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Ref.Box").msgclass
      Ref::Basis = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Ref.Basis").msgclass
      Ref::Project = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Ref.Project").msgclass
      Ref::Target = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Ref.Target").msgclass
      Ref::Machine = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Ref.Machine").msgclass
      Basis = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Basis").msgclass
      Basis::ResourceIdResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Basis.ResourceIdResponse").msgclass
      Basis::DefaultProviderResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Basis.DefaultProviderResponse").msgclass
      Target = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target").msgclass
      Target::ResourceIdResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.ResourceIdResponse").msgclass
      Target::RecordResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.RecordResponse").msgclass
      Target::NameResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.NameResponse").msgclass
      Target::ProjectResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.ProjectResponse").msgclass
      Target::SetNameRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.SetNameRequest").msgclass
      Target::VagrantfileNameResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.VagrantfileNameResponse").msgclass
      Target::VagrantfilePathResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.VagrantfilePathResponse").msgclass
      Target::UpdatedAtResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.UpdatedAtResponse").msgclass
      Target::GetUUIDResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.GetUUIDResponse").msgclass
      Target::SetUUIDRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.SetUUIDRequest").msgclass
      Target::Machine = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.Machine").msgclass
      Target::Machine::BoxResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.Machine.BoxResponse").msgclass
      Target::Machine::SetIDRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.Machine.SetIDRequest").msgclass
      Target::Machine::GetIDResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.Machine.GetIDResponse").msgclass
      Target::Machine::SetStateRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.Machine.SetStateRequest").msgclass
      Target::Machine::GetStateResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.Machine.GetStateResponse").msgclass
      Target::Machine::UIDResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.Machine.UIDResponse").msgclass
      Target::Machine::SyncedFoldersResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.Machine.SyncedFoldersResponse").msgclass
      Target::Machine::SyncedFoldersResponse::Folder = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.Machine.SyncedFoldersResponse.Folder").msgclass
      Target::Machine::SyncedFoldersResponse::MachineSyncedFolder = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Target.Machine.SyncedFoldersResponse.MachineSyncedFolder").msgclass
      Project = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Project").msgclass
      Project::ActiveTargetsResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Project.ActiveTargetsResponse").msgclass
      Project::ConfigResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Project.ConfigResponse").msgclass
      Project::CwdResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Project.CwdResponse").msgclass
      Project::DefaultProviderRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Project.DefaultProviderRequest").msgclass
      Project::DefaultProviderResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Project.DefaultProviderResponse").msgclass
      Project::HomeResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Project.HomeResponse").msgclass
      Project::LocalDataResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Project.LocalDataResponse").msgclass
      Project::PrimaryTargetNameResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Project.PrimaryTargetNameResponse").msgclass
      Project::ResourceIdResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Project.ResourceIdResponse").msgclass
      Project::TargetRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Project.TargetRequest").msgclass
      Project::TargetNamesResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Project.TargetNamesResponse").msgclass
      Project::TargetIdsResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Project.TargetIdsResponse").msgclass
      Project::VagrantfileNameResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Project.VagrantfileNameResponse").msgclass
      Vagrantfile = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile").msgclass
      Vagrantfile::ValueRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.ValueRequest").msgclass
      Vagrantfile::Serialized = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.Serialized").msgclass
      Vagrantfile::BoxCollection = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.BoxCollection").msgclass
      Vagrantfile::NamespaceRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.NamespaceRequest").msgclass
      Vagrantfile::TargetRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.TargetRequest").msgclass
      Vagrantfile::TargetResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.TargetResponse").msgclass
      Vagrantfile::TargetConfigRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.TargetConfigRequest").msgclass
      Vagrantfile::TargetConfigResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.TargetConfigResponse").msgclass
      Vagrantfile::TargetNamesResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.TargetNamesResponse").msgclass
      Vagrantfile::PrimaryTargetNameResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.PrimaryTargetNameResponse").msgclass
      Vagrantfile::ConfigVM = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.ConfigVM").msgclass
      Vagrantfile::ConfigVagrant = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.ConfigVagrant").msgclass
      Vagrantfile::GeneralConfig = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.GeneralConfig").msgclass
      Vagrantfile::MachineConfig = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.MachineConfig").msgclass
      Vagrantfile::Provisioner = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.Provisioner").msgclass
      Vagrantfile::Provider = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.Provider").msgclass
      Vagrantfile::Network = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.Network").msgclass
      Vagrantfile::SyncedFolder = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.SyncedFolder").msgclass
      Vagrantfile::PushConfig = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.PushConfig").msgclass
      Vagrantfile::Vagrantfile = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Vagrantfile.Vagrantfile").msgclass
      TargetIndex = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TargetIndex").msgclass
      TargetIndex::TargetIdentifier = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TargetIndex.TargetIdentifier").msgclass
      TargetIndex::AllResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TargetIndex.AllResponse").msgclass
      TargetIndex::IncludesResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.TargetIndex.IncludesResponse").msgclass
      Box = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Box").msgclass
      Box::AutomaticUpdateCheckAllowedResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Box.AutomaticUpdateCheckAllowedResponse").msgclass
      Box::HasUpdateRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Box.HasUpdateRequest").msgclass
      Box::HasUpdateResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Box.HasUpdateResponse").msgclass
      Box::UpdateInfoResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Box.UpdateInfoResponse").msgclass
      Box::InUseResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Box.InUseResponse").msgclass
      Box::MachinesResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Box.MachinesResponse").msgclass
      Box::BoxMetadataResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Box.BoxMetadataResponse").msgclass
      Box::MetadataUrlResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Box.MetadataUrlResponse").msgclass
      Box::NameResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Box.NameResponse").msgclass
      Box::ProviderResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Box.ProviderResponse").msgclass
      Box::VersionResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Box.VersionResponse").msgclass
      Box::EqualityResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.Box.EqualityResponse").msgclass
      BoxCollection = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxCollection").msgclass
      BoxCollection::AddRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxCollection.AddRequest").msgclass
      BoxCollection::AllResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxCollection.AllResponse").msgclass
      BoxCollection::CleanRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxCollection.CleanRequest").msgclass
      BoxCollection::FindRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxCollection.FindRequest").msgclass
      BoxMetadata = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxMetadata").msgclass
      BoxMetadata::BoxMetadataOpts = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxMetadata.BoxMetadataOpts").msgclass
      BoxMetadata::LoadMetadataRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxMetadata.LoadMetadataRequest").msgclass
      BoxMetadata::NameResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxMetadata.NameResponse").msgclass
      BoxMetadata::VersionQuery = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxMetadata.VersionQuery").msgclass
      BoxMetadata::VersionResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxMetadata.VersionResponse").msgclass
      BoxMetadata::ListVersionsQuery = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxMetadata.ListVersionsQuery").msgclass
      BoxMetadata::ListVersionsResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxMetadata.ListVersionsResponse").msgclass
      BoxMetadata::ProviderRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxMetadata.ProviderRequest").msgclass
      BoxMetadata::ProviderResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxMetadata.ProviderResponse").msgclass
      BoxMetadata::ListProvidersRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxMetadata.ListProvidersRequest").msgclass
      BoxMetadata::ListProvidersResponse = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("hashicorp.vagrant.sdk.BoxMetadata.ListProvidersResponse").msgclass
    end
  end
end
