require_relative "../../../base"

require "vagrant/util/template_renderer"

describe "templates/guests/suse/network_static" do
  let(:template) { "guests/suse/network_static" }

  it "renders the template" do
    result = Vagrant::Util::TemplateRenderer.render(template, options: {
      device:    "eth1",
      ip:        "1.1.1.1",
      netmask:   "255.255.0.0",
    })
    expect(result).to eq <<-EOH.gsub(/^ {6}/, "")
      #VAGRANT-BEGIN
      # The contents below are automatically generated by Vagrant. Do not modify.
      BOOTPROTO='static'
      IPADDR='1.1.1.1'
      NETMASK='255.255.0.0'
      DEVICE='eth1'
      PEERDNS='no'
      STARTMODE='auto'
      USERCONTROL='no'
      #VAGRANT-END
    EOH
  end

  it "includes the gateway" do
    result = Vagrant::Util::TemplateRenderer.render(template, options: {
      device:     "eth1",
      ip:         "1.1.1.1",
      gateway:    "1.2.3.4",
      netmask:    "255.255.0.0",
    })
    expect(result).to eq <<-EOH.gsub(/^ {6}/, "")
      #VAGRANT-BEGIN
      # The contents below are automatically generated by Vagrant. Do not modify.
      BOOTPROTO='static'
      IPADDR='1.1.1.1'
      NETMASK='255.255.0.0'
      DEVICE='eth1'
      GATEWAY='1.2.3.4'
      PEERDNS='no'
      STARTMODE='auto'
      USERCONTROL='no'
      #VAGRANT-END
    EOH
  end
end
