#VAGRANT-BEGIN
# The contents below are automatically generated by Vagrant. Do not modify.
BOOTPROTO='static'
IPADDR='<%= options[:ip] %>'
NETMASK='<%= options[:netmask] %>'
DEVICE='<%= options[:device] %>'
<% if options[:gateway] -%>
GATEWAY='<%= options[:gateway] %>'
<% end -%>
PEERDNS='no'
STARTMODE='auto'
USERCONTROL='no'
#VAGRANT-END
