#!/bin/bash
set -e

# Get the parent directory of where this script is.
SOURCE="${BASH_SOURCE[0]}"
while [ -h "$SOURCE" ] ; do SOURCE="$(readlink "$SOURCE")"; done
DIR="$( cd -P "$( dirname "$SOURCE" )/.." && pwd )"

# Change into that dir because we expect that
cd $DIR

# Get the version from the command line
VERSION=$1
if [ -z $VERSION ]; then
    echo "Please specify a version."
    exit 1
fi

# Make the checksums
pushd ./pkg/dist
shasum -a256 * > ./vagrant_${VERSION}_SHA256SUMS
# if [ -z $NOSIGN ]; then
#   echo "==> Signing..."
#   gpg --default-key 348FFC4C --detach-sig ./vagrant_${VERSION}_SHA256SUMS
# fi
popd
