module VagrantPlugins
  module CommandServe
    # Simple constant aliases to reduce namespace typing
    SDK = Hashicorp::Vagrant::Sdk
    SRV = Hashicorp::Vagrant
    Empty = ::Google::Protobuf::Empty
  end
end
