log_level          <%= log_level.inspect %>
log_location       STDOUT
verbose_logging    <%= verbose_logging.inspect %>
<% if node_name %>
node_name          "<%= node_name %>"
<% end %>
ssl_verify_mode    :verify_none
chef_server_url    "<%= chef_server_url %>"

validation_client_name "<%= validation_client_name %>"
validation_key         "<%= validation_key %>"
client_key             "<%= client_key %>"

encrypted_data_bag_secret <%= encrypted_data_bag_secret.inspect %>

<% if environment %>
environment        "<%= environment %>"
<% end %>

file_cache_path    "<%= file_cache_path %>"
file_backup_path   "<%= file_backup_path %>"

http_proxy <%= http_proxy.inspect %>
http_proxy_user <%= http_proxy_user.inspect %>
http_proxy_pass <%= http_proxy_pass.inspect %>
https_proxy <%= https_proxy.inspect %>
https_proxy_user <%= https_proxy_user.inspect %>
https_proxy_pass <%= https_proxy_pass.inspect %>
no_proxy <%= no_proxy.inspect %>

pid_file           "/var/run/chef/chef-client.pid"

Mixlib::Log::Formatter.show_time = true

<% if formatter %>
add_formatter "<%= formatter %>"
<% end %>

<% if custom_configuration -%>
Chef::Config.from_file "<%= custom_configuration %>"
<% end -%>
