#VAGRANT-BEGIN
ifconfig_<%= options[:device] %>_ipv6="inet6 <%= options[:ip] %> prefixlen <%= options[:netmask] %>"
<% if options[:gateway] %>
ipv6_defaultrouter="<%= options[:gateway] %>"
<% end %>
#VAGRANT-END
