module VagrantPlugins
  module CommandServe
    class Mappers
      module Internal
        class Graph
          # Represents a root vertex within the graph
          class Vertex
            class Root < Vertex
            end
          end
        end
      end
    end
  end
end
