#VAGRANT-BEGIN
# The contents below are automatically generated by Vagrant. Do not modify.
BOOTPROTO=dhcp
ONBOOT=yes
DEVICE=<%= options[:device] %>
NM_CONTROLLED=<%= options.fetch(:nm_controlled, "no") %>
#VAGRANT-END
