#VAGRANT-BEGIN
# The contents below are automatically generated by Vagrant. Do not modify.
template='interface'
ipaddr='<%= options[:ip] %>/<%= options[:netmask] %>'
<% [:gateway, :nameservers, :domain, :route, :gateway6, :route6, :mtu].each do |key| %>
<% if options[key] %>
<%= key %>='<%= options[key] %>'
<% end %>
<% end %>
#VAGRANT-END
