#VAGRANT-BEGIN
# The contents below are automatically generated by Vagrant. Do not modify.
config_<%= options[:device] %>="<%= options[:ip] %>/<%= options[:netmask] %>"
modules_<%= options[:device] %>="!plug"
<% if options[:gateway] -%>
gateways_<%= options[:device] %>="<%= options[:gateway] %>"
<% end -%>
#VAGRANT-END
