#VAGRANT-BEGIN
# The contents below are automatically generated by Vagrant. Do not modify.
auto <%= options[:device] %>
iface <%= options[:device] %> inet6 static
      address <%= options[:ip] %>
      netmask <%= options[:netmask] %>
<% if options[:gateway] %>
      gateway <%= options[:gateway] %>
<% end %>
#VAGRANT-END
