#VAGRANT-BEGIN
# The contents below are automatically generated by Vagrant. Do not modify.
STARTMODE='auto'
BOOTPROTO='static'
IPADDR=<%= options[:ip] %>
<% if options[:netmask] -%>
NETMASK=<%= options[:netmask] %>
<% end -%>
DEVICE=<%= options[:device] %>
<% if options[:gateway] -%>
GATEWAY=<%= options[:gateway] %>
<% end -%>
<% if options[:prefix_length] -%>
PREFIXLEN=<%= options[:prefix_length] %>
<% end -%>
#VAGRANT-END
