#VAGRANT-BEGIN
# The contents below are automatically generated by Vagrant. Do not modify.
auto eth<%= options[:interface] %>
iface eth<%= options[:interface] %> inet dhcp
<% if !options[:use_dhcp_assigned_default_route] %>
    post-up route del default dev $IFACE || true
<% else %>
    # We need to disable eth0, see GH-2648
    post-up route del default dev eth0
    post-up dhclient $IFACE
    pre-down route add default dev eth0
<% end %>
#VAGRANT-END
